# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flatbuf

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

class Tensor(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = Tensor()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsTensor(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # Tensor
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Tensor
    def TypeType(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint8Flags, o + self._tab.Pos)
        return 0

    # The type of data contained in a value cell. Currently only fixed-width
    # value types are supported, no strings or nested types
    # Tensor
    def Type(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            from flatbuffers.table import Table
            obj = Table(bytearray(), 0)
            self._tab.Union(obj, o)
            return obj
        return None

    # The dimensions of the tensor, optionally named
    # Tensor
    def Shape(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from vastdb.vast_flatbuf.org.apache.arrow.flatbuf.TensorDim import TensorDim
            obj = TensorDim()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Tensor
    def ShapeLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # Tensor
    def ShapeIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        return o == 0

    # Non-negative byte offsets to advance one value cell along each dimension
    # If omitted, default to row-major order (C-like).
    # Tensor
    def Strides(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.Get(flatbuffers.number_types.Int64Flags, a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 8))
        return 0

    # Tensor
    def StridesAsNumpy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.GetVectorAsNumpy(flatbuffers.number_types.Int64Flags, o)
        return 0

    # Tensor
    def StridesLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # Tensor
    def StridesIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        return o == 0

    # The location and size of the tensor's data
    # Tensor
    def Data(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            x = o + self._tab.Pos
            from vastdb.vast_flatbuf.org.apache.arrow.flatbuf.Buffer import Buffer
            obj = Buffer()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

def Start(builder): builder.StartObject(5)
def TensorStart(builder):
    """This method is deprecated. Please switch to Start."""
    return Start(builder)
def AddTypeType(builder, typeType): builder.PrependUint8Slot(0, typeType, 0)
def TensorAddTypeType(builder, typeType):
    """This method is deprecated. Please switch to AddTypeType."""
    return AddTypeType(builder, typeType)
def AddType(builder, type): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(type), 0)
def TensorAddType(builder, type):
    """This method is deprecated. Please switch to AddType."""
    return AddType(builder, type)
def AddShape(builder, shape): builder.PrependUOffsetTRelativeSlot(2, flatbuffers.number_types.UOffsetTFlags.py_type(shape), 0)
def TensorAddShape(builder, shape):
    """This method is deprecated. Please switch to AddShape."""
    return AddShape(builder, shape)
def StartShapeVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def TensorStartShapeVector(builder, numElems):
    """This method is deprecated. Please switch to Start."""
    return StartShapeVector(builder, numElems)
def AddStrides(builder, strides): builder.PrependUOffsetTRelativeSlot(3, flatbuffers.number_types.UOffsetTFlags.py_type(strides), 0)
def TensorAddStrides(builder, strides):
    """This method is deprecated. Please switch to AddStrides."""
    return AddStrides(builder, strides)
def StartStridesVector(builder, numElems): return builder.StartVector(8, numElems, 8)
def TensorStartStridesVector(builder, numElems):
    """This method is deprecated. Please switch to Start."""
    return StartStridesVector(builder, numElems)
def AddData(builder, data): builder.PrependStructSlot(4, flatbuffers.number_types.UOffsetTFlags.py_type(data), 0)
def TensorAddData(builder, data):
    """This method is deprecated. Please switch to AddData."""
    return AddData(builder, data)
def End(builder): return builder.EndObject()
def TensorEnd(builder):
    """This method is deprecated. Please switch to End."""
    return End(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flatbuf

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# ----------------------------------------------------------------------
# Arrow File metadata
#
class Footer(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = Footer()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsFooter(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # Footer
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Footer
    def Version(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int16Flags, o + self._tab.Pos)
        return 0

    # Footer
    def Schema(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from vastdb.vast_flatbuf.org.apache.arrow.flatbuf.Schema import Schema
            obj = Schema()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Footer
    def Dictionaries(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 24
            from vastdb.vast_flatbuf.org.apache.arrow.flatbuf.Block import Block
            obj = Block()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Footer
    def DictionariesLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # Footer
    def DictionariesIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        return o == 0

    # Footer
    def RecordBatches(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 24
            from vastdb.vast_flatbuf.org.apache.arrow.flatbuf.Block import Block
            obj = Block()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Footer
    def RecordBatchesLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # Footer
    def RecordBatchesIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        return o == 0

    # User-defined metadata
    # Footer
    def CustomMetadata(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from vastdb.vast_flatbuf.org.apache.arrow.flatbuf.KeyValue import KeyValue
            obj = KeyValue()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Footer
    def CustomMetadataLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # Footer
    def CustomMetadataIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        return o == 0

def Start(builder): builder.StartObject(5)
def FooterStart(builder):
    """This method is deprecated. Please switch to Start."""
    return Start(builder)
def AddVersion(builder, version): builder.PrependInt16Slot(0, version, 0)
def FooterAddVersion(builder, version):
    """This method is deprecated. Please switch to AddVersion."""
    return AddVersion(builder, version)
def AddSchema(builder, schema): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(schema), 0)
def FooterAddSchema(builder, schema):
    """This method is deprecated. Please switch to AddSchema."""
    return AddSchema(builder, schema)
def AddDictionaries(builder, dictionaries): builder.PrependUOffsetTRelativeSlot(2, flatbuffers.number_types.UOffsetTFlags.py_type(dictionaries), 0)
def FooterAddDictionaries(builder, dictionaries):
    """This method is deprecated. Please switch to AddDictionaries."""
    return AddDictionaries(builder, dictionaries)
def StartDictionariesVector(builder, numElems): return builder.StartVector(24, numElems, 8)
def FooterStartDictionariesVector(builder, numElems):
    """This method is deprecated. Please switch to Start."""
    return StartDictionariesVector(builder, numElems)
def AddRecordBatches(builder, recordBatches): builder.PrependUOffsetTRelativeSlot(3, flatbuffers.number_types.UOffsetTFlags.py_type(recordBatches), 0)
def FooterAddRecordBatches(builder, recordBatches):
    """This method is deprecated. Please switch to AddRecordBatches."""
    return AddRecordBatches(builder, recordBatches)
def StartRecordBatchesVector(builder, numElems): return builder.StartVector(24, numElems, 8)
def FooterStartRecordBatchesVector(builder, numElems):
    """This method is deprecated. Please switch to Start."""
    return StartRecordBatchesVector(builder, numElems)
def AddCustomMetadata(builder, customMetadata): builder.PrependUOffsetTRelativeSlot(4, flatbuffers.number_types.UOffsetTFlags.py_type(customMetadata), 0)
def FooterAddCustomMetadata(builder, customMetadata):
    """This method is deprecated. Please switch to AddCustomMetadata."""
    return AddCustomMetadata(builder, customMetadata)
def StartCustomMetadataVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def FooterStartCustomMetadataVector(builder, numElems):
    """This method is deprecated. Please switch to Start."""
    return StartCustomMetadataVector(builder, numElems)
def End(builder): return builder.EndObject()
def FooterEnd(builder):
    """This method is deprecated. Please switch to End."""
    return End(builder)


# automatically generated by the FlatBuffers compiler, do not modify

# namespace: tabular

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# used by `ListSchemas` and `ListTables` API
class ObjectDetails(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = ObjectDetails()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsObjectDetails(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # ObjectDetails
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # ObjectDetails
    def Name(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # ObjectDetails
    def Properties(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # ObjectDetails
    def Handle(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # ObjectDetails
    def NumRows(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint64Flags, o + self._tab.Pos)
        return 0

    # ObjectDetails
    def SizeInBytes(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint64Flags, o + self._tab.Pos)
        return 0

    # ObjectDetails
    def NumPartitions(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint64Flags, o + self._tab.Pos)
        return 0

    # ObjectDetails
    def SortingKeyEnabled(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(16))
        if o != 0:
            return bool(self._tab.Get(flatbuffers.number_types.BoolFlags, o + self._tab.Pos))
        return False

    # ObjectDetails
    def SortingScore(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(18))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int64Flags, o + self._tab.Pos)
        return 0

    # ObjectDetails
    def WriteAmplification(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(20))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int64Flags, o + self._tab.Pos)
        return 0

    # ObjectDetails
    def AcummulativeRowInseritionCount(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(22))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int64Flags, o + self._tab.Pos)
        return 0

def ObjectDetailsStart(builder):
    builder.StartObject(10)

def Start(builder):
    ObjectDetailsStart(builder)

def ObjectDetailsAddName(builder, name):
    builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(name), 0)

def AddName(builder, name):
    ObjectDetailsAddName(builder, name)

def ObjectDetailsAddProperties(builder, properties):
    builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(properties), 0)

def AddProperties(builder, properties):
    ObjectDetailsAddProperties(builder, properties)

def ObjectDetailsAddHandle(builder, handle):
    builder.PrependUOffsetTRelativeSlot(2, flatbuffers.number_types.UOffsetTFlags.py_type(handle), 0)

def AddHandle(builder, handle):
    ObjectDetailsAddHandle(builder, handle)

def ObjectDetailsAddNumRows(builder, numRows):
    builder.PrependUint64Slot(3, numRows, 0)

def AddNumRows(builder, numRows):
    ObjectDetailsAddNumRows(builder, numRows)

def ObjectDetailsAddSizeInBytes(builder, sizeInBytes):
    builder.PrependUint64Slot(4, sizeInBytes, 0)

def AddSizeInBytes(builder, sizeInBytes):
    ObjectDetailsAddSizeInBytes(builder, sizeInBytes)

def ObjectDetailsAddNumPartitions(builder, numPartitions):
    builder.PrependUint64Slot(5, numPartitions, 0)

def AddNumPartitions(builder, numPartitions):
    ObjectDetailsAddNumPartitions(builder, numPartitions)

def ObjectDetailsAddSortingKeyEnabled(builder, sortingKeyEnabled):
    builder.PrependBoolSlot(6, sortingKeyEnabled, 0)

def AddSortingKeyEnabled(builder, sortingKeyEnabled):
    ObjectDetailsAddSortingKeyEnabled(builder, sortingKeyEnabled)

def ObjectDetailsAddSortingScore(builder, sortingScore):
    builder.PrependInt64Slot(7, sortingScore, 0)

def AddSortingScore(builder, sortingScore):
    ObjectDetailsAddSortingScore(builder, sortingScore)

def ObjectDetailsAddWriteAmplification(builder, writeAmplification):
    builder.PrependInt64Slot(8, writeAmplification, 0)

def AddWriteAmplification(builder, writeAmplification):
    ObjectDetailsAddWriteAmplification(builder, writeAmplification)

def ObjectDetailsAddAcummulativeRowInseritionCount(builder, acummulativeRowInseritionCount):
    builder.PrependInt64Slot(9, acummulativeRowInseritionCount, 0)

def AddAcummulativeRowInseritionCount(builder, acummulativeRowInseritionCount):
    ObjectDetailsAddAcummulativeRowInseritionCount(builder, acummulativeRowInseritionCount)

def ObjectDetailsEnd(builder):
    return builder.EndObject()

def End(builder):
    return ObjectDetailsEnd(builder)



import logging
from pathlib import Path
from threading import RLock
from typing import Optional, Union

from vastdb.bench.perf_bench.common.constants import LogLevel
from vastdb.bench.perf_bench.common.types import PathLikeT


class LogConfigError(Exception):
    pass


_logging_configured: bool = False
_log_level: int = logging.INFO
_log_file: Optional[Path] = None
_lock: RLock = RLock()


def set_log_file(log_file: PathLikeT) -> None:
    global _log_file  # noqa: PLW0603
    with _lock:
        if _logging_configured:
            raise LogConfigError(
                "Cannot change log file after logging has been configured."
            )

        log_file = Path(str(log_file))
        log_file.parent.mkdir(parents=True, exist_ok=True)
        log_file.touch(exist_ok=True)
        _log_file = log_file


def get_log_level() -> int:
    return _log_level


def set_log_level(level: Union[str, LogLevel, int]) -> None:
    global _log_level  # noqa: PLW0603
    if isinstance(level, str):
        level = LogLevel[level].to_int()
    with _lock:
        if level != _log_level and _logging_configured:
            raise LogConfigError(
                "Cannot change log level after logging has been configured."
            )
        _log_level = level


def get_logger(name: Optional[str]) -> logging.Logger:
    global _logging_configured  # noqa: PLW0603
    with _lock:
        if not _logging_configured:
            logging.basicConfig(
                level=_log_level,
                format=(
                    "%(asctime)s.%(msecs)03d %(levelname)s %(module)s - %(funcName)s: %(message)s"
                ),
                datefmt="%Y-%m-%d %H:%M:%S",
            )
            if _log_file:
                root_logger = logging.getLogger()
                fh = logging.FileHandler(_log_file, mode="a")
                fh.setFormatter(root_logger.handlers[0].formatter)
                root_logger.addHandler(fh)
            _logging_configured = True
    return logging.getLogger(name)



import logging

import duckdb
import pyarrow as pa
import pyarrow.compute as pc
import pytest

from ..table import QueryConfig
from .util import prepare_data

log = logging.getLogger(__name__)


def test_duckdb(session, clean_bucket_name):
    columns = pa.schema([
        ('a', pa.int32()),
        ('b', pa.float64()),
    ])
    data = pa.table(schema=columns, data=[
        [111, 222, 333],
        [0.5, 1.5, 2.5],
    ])
    with prepare_data(session, clean_bucket_name, 's', 't', data) as t:
        conn = duckdb.connect()
        batches = t.select(columns=['a'], predicate=(t['b'] < 2))  # noqa: F841
        actual = conn.execute('SELECT max(a) as "a_max" FROM batches').arrow()
        expected = (data
            .filter(pc.field('b') < 2)
            .group_by([])
            .aggregate([('a', 'max')]))
        assert actual == expected


def test_closed_tx(session, clean_bucket_name):
    columns = pa.schema([
        ('a', pa.int64()),
    ])
    data = pa.table(schema=columns, data=[
        list(range(10000)),
    ])

    with session.transaction() as tx:
        t = tx.bucket(clean_bucket_name).create_schema("s1").create_table("t1", columns)
        t.insert(data)

        config = QueryConfig(
            num_sub_splits=1,
            num_splits=1,
            num_row_groups_per_sub_split=1,
            limit_rows_per_sub_split=100)
        batches = t.select(config=config)  # noqa: F841
        first = next(batches)  # make sure that HTTP response processing has started
        assert first['a'].to_pylist() == list(range(100))

        conn = duckdb.connect()
        res = conn.execute('SELECT a FROM batches')
        log.debug("closing tx=%s after first batch=%s", t.tx, first)

    # transaction is closed, collecting the result should fail internally in DuckDB
    with pytest.raises(duckdb.InvalidInputException):
        res.arrow()



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flatbuf

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# Struct field access
class StructField(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = StructField()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsStructField(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # StructField
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # The position of the field in the struct schema
    # StructField
    def Position(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint32Flags, o + self._tab.Pos)
        return 0

def Start(builder): builder.StartObject(1)
def StructFieldStart(builder):
    """This method is deprecated. Please switch to Start."""
    return Start(builder)
def AddPosition(builder, position): builder.PrependUint32Slot(0, position, 0)
def StructFieldAddPosition(builder, position):
    """This method is deprecated. Please switch to AddPosition."""
    return AddPosition(builder, position)
def End(builder): return builder.EndObject()
def StructFieldEnd(builder):
    """This method is deprecated. Please switch to End."""
    return End(builder)


import dataclasses
import sqlite3
from typing import List

_MAP_SQLITE_TYPES = {
    str: "TEXT",
    float: "REAL",
    int: "INTEGER",
}


@dataclasses.dataclass
class Row:
    start: float
    finish: float
    table_path: str
    op: str
    nbytes: int
    rows: int
    cols: int
    pid: int
    tid: int
    sdk_version: str


class Table:
    def __init__(self, conn: sqlite3.Connection, name: str):
        self.fields = dataclasses.fields(Row)
        self.conn = conn
        self.name = name
        columns = ", ".join(
            f"{f.name} {_MAP_SQLITE_TYPES[f.type]}"  # type: ignore
            for f in self.fields
        )
        cmd = f"CREATE TABLE {self.name} ({columns})"
        self.conn.execute(cmd).fetchall()

    def insert(self, rows: List[Row]):
        args = ", ".join(["?"] * len(self.fields))
        cmd = f"INSERT INTO {self.name} VALUES ({args})"
        data = [dataclasses.astuple(row) for row in rows]
        self.conn.executemany(cmd, data).fetchall()
        self.conn.commit()



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flatbuf

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# ----------------------------------------------------------------------
# EXPERIMENTAL: Data structures for sparse tensors
# Coordinate (COO) format of sparse tensor index.
#
# COO's index list are represented as a NxM matrix,
# where N is the number of non-zero values,
# and M is the number of dimensions of a sparse tensor.
#
# indicesBuffer stores the location and size of the data of this indices
# matrix.  The value type and the stride of the indices matrix is
# specified in indicesType and indicesStrides fields.
#
# For example, let X be a 2x3x4x5 tensor, and it has the following
# 6 non-zero values:
# ```text
#   X[0, 1, 2, 0] := 1
#   X[1, 1, 2, 3] := 2
#   X[0, 2, 1, 0] := 3
#   X[0, 1, 3, 0] := 4
#   X[0, 1, 2, 1] := 5
#   X[1, 2, 0, 4] := 6
# ```
# In COO format, the index matrix of X is the following 4x6 matrix:
# ```text
#   [[0, 0, 0, 0, 1, 1],
#    [1, 1, 1, 2, 1, 2],
#    [2, 2, 3, 1, 2, 0],
#    [0, 1, 0, 0, 3, 4]]
# ```
# When isCanonical is true, the indices is sorted in lexicographical order
# (row-major order), and it does not have duplicated entries.  Otherwise,
# the indices may not be sorted, or may have duplicated entries.
class SparseTensorIndexCOO(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = SparseTensorIndexCOO()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsSparseTensorIndexCOO(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # SparseTensorIndexCOO
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # The type of values in indicesBuffer
    # SparseTensorIndexCOO
    def IndicesType(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from vastdb.vast_flatbuf.org.apache.arrow.flatbuf.Int import Int
            obj = Int()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Non-negative byte offsets to advance one value cell along each dimension
    # If omitted, default to row-major order (C-like).
    # SparseTensorIndexCOO
    def IndicesStrides(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.Get(flatbuffers.number_types.Int64Flags, a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 8))
        return 0

    # SparseTensorIndexCOO
    def IndicesStridesAsNumpy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.GetVectorAsNumpy(flatbuffers.number_types.Int64Flags, o)
        return 0

    # SparseTensorIndexCOO
    def IndicesStridesLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # SparseTensorIndexCOO
    def IndicesStridesIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        return o == 0

    # The location and size of the indices matrix's data
    # SparseTensorIndexCOO
    def IndicesBuffer(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            x = o + self._tab.Pos
            from vastdb.vast_flatbuf.org.apache.arrow.flatbuf.Buffer import Buffer
            obj = Buffer()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # This flag is true if and only if the indices matrix is sorted in
    # row-major order, and does not have duplicated entries.
    # This sort order is the same as of Tensorflow's SparseTensor,
    # but it is inverse order of SciPy's canonical coo_matrix
    # (SciPy employs column-major order for its coo_matrix).
    # SparseTensorIndexCOO
    def IsCanonical(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return bool(self._tab.Get(flatbuffers.number_types.BoolFlags, o + self._tab.Pos))
        return False

def Start(builder): builder.StartObject(4)
def SparseTensorIndexCOOStart(builder):
    """This method is deprecated. Please switch to Start."""
    return Start(builder)
def AddIndicesType(builder, indicesType): builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(indicesType), 0)
def SparseTensorIndexCOOAddIndicesType(builder, indicesType):
    """This method is deprecated. Please switch to AddIndicesType."""
    return AddIndicesType(builder, indicesType)
def AddIndicesStrides(builder, indicesStrides): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(indicesStrides), 0)
def SparseTensorIndexCOOAddIndicesStrides(builder, indicesStrides):
    """This method is deprecated. Please switch to AddIndicesStrides."""
    return AddIndicesStrides(builder, indicesStrides)
def StartIndicesStridesVector(builder, numElems): return builder.StartVector(8, numElems, 8)
def SparseTensorIndexCOOStartIndicesStridesVector(builder, numElems):
    """This method is deprecated. Please switch to Start."""
    return StartIndicesStridesVector(builder, numElems)
def AddIndicesBuffer(builder, indicesBuffer): builder.PrependStructSlot(2, flatbuffers.number_types.UOffsetTFlags.py_type(indicesBuffer), 0)
def SparseTensorIndexCOOAddIndicesBuffer(builder, indicesBuffer):
    """This method is deprecated. Please switch to AddIndicesBuffer."""
    return AddIndicesBuffer(builder, indicesBuffer)
def AddIsCanonical(builder, isCanonical): builder.PrependBoolSlot(3, isCanonical, 0)
def SparseTensorIndexCOOAddIsCanonical(builder, isCanonical):
    """This method is deprecated. Please switch to AddIsCanonical."""
    return AddIsCanonical(builder, isCanonical)
def End(builder): return builder.EndObject()
def SparseTensorIndexCOOEnd(builder):
    """This method is deprecated. Please switch to End."""
    return End(builder)


import datetime as dt
import os
import sys
from typing import Union

PathLikeT = Union[str, bytes, os.PathLike]
DateLikeT = Union[str, dt.date]

if sys.version_info >= (3, 11):
    # noinspection PyUnresolvedReferences
    from enum import StrEnum
else:
    from enum import Enum, auto

    class StrEnum(str, Enum):
        # noinspection PyTypeChecker
        def __new__(cls, value: Union[auto, str], *args, **kwargs):
            if not isinstance(value, (str, auto)):
                raise TypeError(
                    f"Not a string/auto type: {value=!r} [type={type(value)}]"
                )
            return super().__new__(cls, value, *args, **kwargs)

        def __repr__(self):
            """Return a string representation of the enumeration member."""
            return f"<{self.__class__.__name__}.{self.name}: '{self.value}'>"

        def __str__(self):
            """Return the value of the enumeration member."""
            return str(self.value)

        @staticmethod
        def _generate_next_value_(name: str, *args, **kwargs):
            return name



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flatbuf

# Whether lesser values should precede greater or vice versa,
# also whether nulls should preced or follow values
class Ordering(object):
    ASCENDING_THEN_NULLS = 0
    DESCENDING_THEN_NULLS = 1
    NULLS_THEN_ASCENDING = 2
    NULLS_THEN_DESCENDING = 3

